C
C -------------------------------------------------------------------
C   FORTRAN 77 collection of graphics routines which can be substituted
C   for CalCOMP graphic software calls.  The following routines are
C   included:
C      SUBROUTINE AXIS
C      SUBROUTINE SCALE
C      SUBROUTINE NUMBER
C      SUBROUTINE CURVE
C      SUBROUTINE POLYGON
C
C   In general, these follow  the behavior described in a CALCOMP
C   manual except where some have been modified to handle special uses.
C -------------------------------------------------------------------
C
C axis
C
      SUBROUTINE AXIS(X,Y,LBL,NC,S,THETA,SMIN,DS)
C
C     + + + PURPOSE + + +
C     Draws an axis, annotates the divisions with appropriate scale
C     values, and draws a centered title.  When both X and Y axes are
C     required, AXIS must be called twice.  Wworks only at even multiples
C     of 90 degrees
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NC
      REAL X, Y, S, THETA, SMIN, DS
      CHARACTER LBL*(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     X,Y   - Coordinates in inches of the starting point of the axis.
C     LBL   - Character title of the axis.
C     NC    - Number of characters in LBL:
C             >0  Annotation and labeling are outside the axis in
C                 relation to the plot.
C             =0  No labeling; annotations are outside the axis.
C             <0  Annotation and labeling are on the inside of the axis
C                 in relation to the plot.
C     S     - Length of the axis in inches.
C     THETA - Angle in degrees of the axis.
C     SMIN  - Value of the first point of the axis.
C             If computed by a main program call to subroutine SCALE,
C             this should be X(N*ABS(INC)+1). If = 999., all labeling
C             is suppressed.
C     DS    - Value of increment added to SMIN for each succeeding one inch
C             division along the axis.  Value may be determined by the main
C             program or computed by a main program call to subroutine SCALE,
C             and referenced in the call to axis as X(N*ABS(INC)+ABS(INC)+1).
C             If SMIN = 999., DS is the distance between tic marks in inches.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER iadj, nac, n, i, nn
      REAL xa, ya, xb, yb, xc, yc, xd, yd, thetb, fac, f, g, dsabs, 
     &     dsadj, shftx, shfty, t, sgn, gsv, fsv, ymin, yyd, sizlb
C
C     + + + EXTERNALS + + +
      EXTERNAL NUMBER, WHERE, SYMBOL, PLOT
C
C     + + + INTRINSICS + + +
      INTRINSIC ISIGN, IABS, FLOAT, ABS, IFIX, ALOG10, AMOD
C
C     + + + END SPECIFICATIONS + + +
C
C---- XB,YB are coordinates of axis end of tic marks:
      sizlb = 0.17
      nn = 1
      xb = X
      yb = Y
      thetb = THETA
      ymin = SMIN
      iadj = 0
      dsadj = DS
      IF (ABS(dsadj).LT.0.00001) dsadj = 0.01
      dsabs = ABS(dsadj)
C
C---- Adjust scaling --
C        less than .01 -- multiply by power of 10
C        greater than 100 -- divide by 10:
C
      IF (dsabs.LT..01 .AND. dsabs.NE.0.0) THEN
        iadj = IFIX(ALOG10(dsabs)+1.00001)
        dsabs = dsabs*10.0**(-iadj)
      ELSEIF (dsabs.GE.100.) THEN
        iadj = IFIX(ALOG10(dsabs)-1.0)
        dsabs = dsabs/10.0**iadj
      ENDIF
      dsadj = DS*10.**(-iadj)
      ymin = SMIN*10.**(-iadj)
C
C---- Ready to begin:
      n = IFIX(S+0.5)
      nac = IABS(NC)
      yyd = .5*(S-FLOAT(nac)*sizlb)
C
      thetb = AMOD(thetb,360.0)
      IF (thetb.LT.45.) THEN
C
C---- 0 degrees:
        f = 1.0
        g = 0.
      ELSEIF (thetb.LT.135.) THEN
C
C---- 90 degrees:
        f = 0.
        g = 1.
      ELSEIF (thetb.LT.225.) THEN
C
C---- 180 degrees:
        f = -1.
        g = 0.
      ELSE
C
C---- 270 degrees:
        f = 0.
        g = -1.
      ENDIF
C
      sgn = ISIGN(1,NC)
      fsv = sgn*f
      gsv = sgn*g
      t = -.1*(sgn-1.)
      shftx = t*gsv
      shfty = t*fsv
C
C---- XA,YA are coordinates of outboard end of tic marks:
      xa = xb - .1*gsv
      ya = yb + .1*fsv
C
C  XC,YC are left hand coordinates of numeric labels:
      xc = xb - sizlb*gsv - sizlb*f - shftx
      yc = yb + sizlb*fsv - sizlb*g + shfty
      xd = xb + yyd*f - .60*gsv - shftx
      yd = yb + yyd*g + .60*fsv + shfty
      IF (SMIN.NE.999.) CALL NUMBER(xc,yc,sizlb,ymin,thetb,nn)
      DO 10 i = 1, n
        CALL PLOT(xa,ya,3)
        CALL PLOT(xb,yb,2)
        xa = xa + f
        xb = xb + f
        xc = xc + f
        ya = ya + g
        yb = yb + g
        yc = yc + g
        ymin = ymin + dsadj
        CALL PLOT(xb,yb,2)
        CALL PLOT(xa,ya,2)
        IF (SMIN.NE.999.) CALL NUMBER(xc,yc,sizlb,ymin,thetb,nn)
   10 CONTINUE
C
C---- Add axis label:
      CALL SYMBOL(xd,yd,sizlb,LBL,thetb,nac)
      IF (iadj.NE.0) THEN
C
C---- Add power of 10 scale adjustment to title
        CALL WHERE(xa,ya,fac)
        xa = xa
        ya = ya
        xa = xa + .1*f
        ya = ya + .1*g
        CALL SYMBOL(xa,ya,sizlb,'*10',thetb,3)
        CALL WHERE(xa,ya,fac)
        xa = xa
        ya = ya
        xa = xa + .02*f - .125*g
        ya = ya + .02*g + .125*f
        CALL NUMBER(xa,ya,sizlb*.5,FLOAT(iadj),thetb,-1)
      ENDIF
C
      RETURN
      END
C
C scale
C
      SUBROUTINE SCALE(X,S,N,INC)
C
C     + + + PURPOSE + + +
C     This subroutine, called by a main program, determines scaling
C     information for the AXIS and LINE subroutines by examining the data
C     values in an array and determining a starting value and a scaling
C     factor such that:  (1) scale annotation drawn by the AXIS subroutine
C     properly represents the range of real data values in the array,
C     (2) the data points, when plotted by the LINE subroutine, fit in a
C     given plotting area, and (3) the values are selected in a way that
C     makes axis annotation easily readable.  The two values, which
C     correspond to SMIN and DS in subroutine AXIS below, are stored by
C     by SCALE at the end of the array X.  The computed minimum (or maximum
C     if INC is negative) value of the array is stored in X(N*ABS(INC)+1).
C     The scale factor is stored in X(N*ABS(INC)+ABS(INC)+1) and will be
C     negative if INC is negative.  X must be dimensioned large enough to
C     contain these two values.  This subroutine needs to be called twice,
C     once for the x-axis array of values, and again for the y-axis array
C     of values.
C
C     + + + DUMMY ARGUMENTS + + +
      REAL X(*), S
      INTEGER N, INC
C
C     + + + ARGUMENT DEFINITIONS + + +
C     X   - A single subscripted array of values (either x-axis or y-axis)
C           that are to fit in the length of the scale provided. (Input/Output)
C     S   - The length of the scale in inches (the axis length over which the
C           data is scaled).  (Input)
C     N   - The number of points to be scanned in array X.  (Input)
C     INC - The interval within X to scan points (the repeat cycle for the
C           array); every INC the point is scanned.  Also, if INC is
C           negative, scale direction is reversed -- YMIN is set to be the
C           maximum value on the scale and DY is a negative increment.
C           This argument is used to skip items in a mixed array.  The actual
C           items scanned by SCALE are:  X(1), X(1+ABS(INC)), X(1+ABS(INC)*2),
C           ..., X(1+ABS(INC)*(N-1).  (Input)
C
C     + + + LOCAL VARIABLES + + +
      REAL dy, zmin, ymax, scfact, tst(9), ymin, xx
      INTEGER incabs, npts, i, n1, n2
C
C     + + + LOCAL VARIABLE DEFINITIONS + + +
C     YMIN = The smallest value to be plotted on the scale.
C     DY   = The scale increment per inch.
C            DY takes on the following values to produce a convenient scale:
C            DY = K * EXP(M)
C            where K = 1, 1.5, 2, 3, 4, 5, 6, 8, OR 10
C            and EXP(M) is the base 10 order of magnitude.
C            The value of each of the values will be plotted at
C            a position XP = (X - YMIN) / DY.
C
C     + + + INTRINSICS + + +
      INTRINSIC ALOG10, AINT, IABS
C
C     + + + END SPECIFICATIONS + + +
C
C---Find maximum and minimum values of array:
      zmin = X(1)
      ymax = zmin
      incabs = IABS(INC)
      npts = N*incabs
      DO 10 i = 1, npts, incabs
        xx = X(i)
        IF (ymax.LT.xx) THEN
          ymax = xx
        ELSEIF (zmin.GT.xx) THEN
          zmin = xx
        ENDIF
   10 CONTINUE
C
C---Special case when the minimum equals the maximum:
      IF (zmin.EQ.ymax) THEN
        xx = S/3.0
        ymax = ymax + xx
        zmin = zmin - xx
      ENDIF
C
C---Find the exponent size:
      dy = (ymax-zmin)/S
C
C---Multiply log by 1.000001 because log(10) is sometimes approximated
C---as .999999:
      ymin = 1.000001*ALOG10(dy)
      IF (dy.LT.1.) THEN
        ymin = AINT(ymin+100.) - 100.
      ELSE
        ymin = AINT(ymin)
      ENDIF
      scfact = 10.**ymin*1.000001
C
C---Find integral magnitude:
      dy = dy/scfact
      tst(1) = 0.10
      tst(2) = 0.15
      tst(3) = 0.20
      tst(4) = 0.25
      tst(5) = 0.30
      tst(6) = 0.40
      tst(7) = 0.50
      tst(8) = 0.60
      tst(9) = 0.80
   20 DO 30 i = 1, 9
        IF (dy.LE.tst(i)) THEN
C
C---Adjust scales to assure fit:
          dy = scfact*tst(i)
          ymin = AINT(zmin/dy)*dy
          IF (zmin.LT.ymin) ymin = ymin - dy
          IF (S.GE.(ymax-ymin)/dy) GOTO 40
        ENDIF
        tst(i) = tst(i)*10.0
   30 CONTINUE
      GOTO 20
C
C---Put results at end of X array:
   40 n1 = npts + 1
      n2 = n1 + incabs
      IF (INC.GT.0) THEN
        X(n1) = ymin
        X(n2) = dy
      ELSE
        X(n1) = ymin + S*dy
        X(n2) = -dy
      ENDIF
C
      RETURN
      END
C
C number
C
      SUBROUTINE NUMBER(X,Y,HEIGHT,FPN,ANGLE,NDEC)
C
C     + + + PURPOSE + + +
C     Plot real variables as character strings, rounded to the
C     specified decimal place.
C
C     + + +  DUMMY ARGUMENTS + + +
      INTEGER NDEC
      REAL X, Y, HEIGHT, FPN, ANGLE
C
C     + + + INPUT ARGUMENT DEFINITIONS + + +
C     X,Y    - The coordinates of the lower left corner of the number.
C     HEIGHT - The height of the number to be plotted.
C     FPN    - The floating point number to be plotted.
C     ANGLE  - The angle in degrees from the x-axis, at which the
C              number is to be plotted.  At ANGLE=0.0, the
C              number is right side up and parallel to the x-axis.
C     NDEC   - The number of digits to the right of the decimal point to
C              be included.
C              If > 0, NDEC is the number of digits to the right of
C                      the decimal point.
C              If = 0, number plotted is an integer plus a decimal point.
C              If = -1, number plotted is an integer with no decimal point.
C              If < -1, ABS(NDEC)-1 digits are truncated from the lower
C                       portion of integer after rounding.  (For example,
C                       if NDEC=-2 for FNUM=127.8 the routine plots 13).
C
C     + + + LOCAL VARIABLES + + +
      CHARACTER string*16, iform*8, cdgt*2
      REAL fnum, pow
      INTEGER nd, nde, i
C
C     + + + LOCAL DEFINITIONS + + +
C     ND       -The number of characters to be plotted.
C     STRING   -The string of characters to be plotted.
C
C     + + + EXTERNALS + + +
      EXTERNAL SYMBOL
C
C     + + + INTRINSICS + + +
      INTRINSIC FLOAT, IFIX, SIGN
C
C     + + + END SPECIFICATIONS + + +
C
C---Convert the real number to a string:
      fnum = FPN
C
C---ND is the number of digits to right of decimal point to retain --
C---always GE 0, NDE is set to -1 if no decimal point.
      IF (NDEC.LT.0) THEN
        nd = 0
        nde = -1
        IF (NDEC.LT.-1) THEN
C
C---Round to power of 10
          pow = 10.**(-NDEC-1)
          fnum = FLOAT(IFIX((fnum+SIGN(.5*pow,fnum))/pow))*pow
        ENDIF
      ELSE
        nd = NDEC
        nde = 0
      ENDIF
C
C---Setup format for encoding FNUM
      iform = '(F16.02)'
      WRITE (cdgt,'(I2)') nd
      iform(6:7) = cdgt
      WRITE (string,iform) fnum
C
C---Scan characters for 1st nonblank digit, ie don't plot leading blanks.
      DO 10 i = 1, 16
        IF (string(i:i).NE.' ') THEN
C
C---Write the string
          nd = 16 + nde
          nde = 17 - i + nde
          CALL SYMBOL(X,Y,HEIGHT,string(i:nd),ANGLE,nde)
          RETURN
        ENDIF
   10 CONTINUE
      END
C
C curve
C
      SUBROUTINE CURVE(BSPC,BDSH,I,NOPTS,XVAL,YVAL)
C
C     + + + PURPOSE + + +
C     This subroutine will conect a series of points to form a curve or line.
C     Line type can be dashed or dotted.
C
C     + + + HISTORY + + +
C     Added by Amy Wheeless July 19, 1993
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER I, NOPTS
      REAL BSPC, BDSH, XVAL(*), YVAL(*)
C
C     + + + ARGUMENT DEFINITIONS + + +
C     BSPC  - Length of spaces between dashes (in inches)
C     BDSH  - Length of dashes (in inches)
C     I     - Subscript of the first point.
C     NOPTS - Number of points.
C     XVAL  - Array of x-coor.'s of points.
C     YVAL  - ARRAY of y-coor.'s of points.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER il, il1, il2
      REAL xl2, yl2, xl1, yl1, xdl, ydl, dis, dyl, oxl, oyl, 
     &     xl3, yl3, rdsh, rspc, ang, dxl
C
C     + + + LOCAL DEFINITIONS + + +
C     il,il1,il2 - loop control variables
C     xl1,yl1 - Starting point of segment
C     xl2,yl2 - Ending point of segment
C     xl3,yl3 - The point on the segment to draw the dash to
C     xdl     - X-length of segment
C     ydl     - Y-length of segment
C     ydl     - Y-length of segment
C     dxl     - X-length of dash
C     dyl     - Y-length of dash
C     odl     - X-length of space
C     oyl     - Y-length of space
C     dis     - The length of a segment
C     ang     - The angle of a segment
C     rdsh    - The remaining dash to start the next segment with.
C     rspc    - The remaining space to start the next segment with.
C
C     + + + EXTERNALS + + +
      EXTERNAL PLOT
C
C     + + + INTRINSICS + + +
      INTRINSIC SQRT, ATAN, COS, SIN, SIGN
C
C     + + + END SPECIFICATIONS + + +
C
      rdsh = 0.0
      rspc = 0.0
      xl2 = XVAL(I)
      yl2 = YVAL(I)
      il1 = I + 1
      il2 = I + NOPTS - 1
      DO 30 il = il1, il2
        xl1 = xl2
        yl1 = yl2
        xl2 = XVAL(il)
        yl2 = YVAL(il)
        xdl = xl2 - xl1
        ydl = yl2 - yl1
        dis = SQRT(xdl*xdl+ydl*ydl)
        ang = ATAN(ydl/xdl)
        IF (dis.LT.1.0E-6) GOTO 30
C---Start this segment with left over space from previous segment.
        IF (rspc.GT.1.0E-6) THEN
          dis = dis - rspc
          IF (dis.GT.1.0E-6) THEN
            xl1 = xl1 + rspc*COS(ang)
            yl1 = yl1 + rspc*SIN(ang)
            rspc = 0.0
          ELSE
C    --- If there is still left over space go to next point.
            rspc = SIGN(dis,1.0)
            GOTO 30
          ENDIF
C---Start this segment with left over dash from previous segment.
        ELSEIF (rdsh.GT.1.0E-6) THEN
          dis = dis - rdsh
          IF (dis.GT.1.0E-6) THEN
            xl3 = xl1 + rdsh*COS(ang)
            yl3 = yl1 + rdsh*SIN(ang)
            CALL PLOT(xl1,yl1,3)
            CALL PLOT(xl3,yl3,2)
            rdsh = 0.0
            GOTO 20
          ELSE
C---If there is still left over dash go to next point.
            rdsh = SIGN(dis,1.0)
            CALL PLOT(xl1,yl1,3)
            CALL PLOT(xl2,yl2,2)
            GOTO 30
          ENDIF
        ENDIF
C---Draw the dash portion.
   10   dxl = BDSH*COS(ang)
        dyl = BDSH*SIN(ang)
        xl3 = xl1 + dxl
        yl3 = yl1 + dyl
        dis = dis - BDSH
        IF (dis.GT.1.0E-6) THEN
C---Draw complete dash to point L3.
          rdsh = 0.0
          CALL PLOT(xl1,yl1,3)
          CALL PLOT(xl3,yl3,2)
        ELSE
C---Draw portion of dash that will fit, go to next point.
          rdsh = SIGN(dis,1.0)
          rspc = 0.0
          CALL PLOT(xl1,yl1,3)
          CALL PLOT(xl2,yl2,2)
          GOTO 30
        ENDIF
C---Leave the space portion.
   20   oxl = BSPC*COS(ang)
        oyl = BSPC*SIN(ang)
        xl1 = xl3 + oxl
        yl1 = yl3 + oyl
        dis = dis - BSPC
        IF (dis.GT.1.0E-6) THEN
C---Leave full space and go to dash portion.
          rspc = 0.0
          GOTO 10
        ENDIF
C---Leave portion of space and go to next point.
        rspc = SIGN(dis,1.0)
        rdsh = 0.0
   30 CONTINUE
C
      RETURN
      END
C
C polygon
C
      SUBROUTINE POLYGON(XC,YC,NSIDES,DIA,ANGLE)
C
C     + + + PURPOSE + + +
C     This subroutine draws an open or closed regular polygon or circle.
C     A circle is drawn as a polygon with a large number of sides of
C     length 0.005 inches.
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER NSIDES
      REAL XC, YC, DIA, ANGLE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     XC     - x-coordinate of the center of the polygon
C     YC     - y-coordinate of the center of the polygon
C     NSIDES - number of sides of the polygon, zero for a circle
C     DIA    - diameter of the circle circumscribed around the polygon,
C              in inches
C     ANGLE  - angle from the x-axis to the first vertex of the polygon
C              (in degrees)
C
C     + + + LOCAL VARIABLES + + +
      INTEGER ntimes, nstart, ncord, i, npt, j
      REAL rad, angle0, cosan, sinan, step, cosst, sinst, x0, y0, x, y, 
     &     x1, y1
C
C     + + + LOCAL VARIABLE DEFINITIONS + + +
C     ntimes - number of different radii for which the polygon is drawn
C     nstart - index corresponding to the first polygon to be drawn
C     ncord  - number of sides of the polygon
C     rad    - radius of the circle circumscribed about the polygon in inches
C     step   - angle of the circle subtended by one side of the polygon
C     x0     - x-coordinate of the starting vertex of the polygon
C     y0     - y-coordinate of the starting vertex of the polygon
C     x      - x-coordinate of the first end of a side of the polygon
C     y1     - y-coordinate of the second end of a side of the polygon
C     x1     - x-coordinate of the second end of a side of the polygon
C     y      - y-coordinate of the first end of a side of the polygon
C
C     + + + EXTERNALS + + +
      EXTERNAL PLOT
C
C     + + + INTRINSICS + + +
      INTRINSIC ABS, INT, COS, SIN, ATAN2, FLOAT
C
C     + + + END SPECIFICATIONS + + +
C
      rad = ABS(DIA)/2.0
      ntimes = INT(rad/0.01+0.5)
      nstart = 1
      IF (DIA.LT.0.0) nstart = ntimes
      angle0 = ANGLE*6.28319/360.
      cosan = COS(angle0)
      sinan = SIN(angle0)
      IF (NSIDES.EQ.0) THEN
        step = ATAN2(0.005,rad)
        ncord = INT(6.28319/step)
      ELSE
        ncord = NSIDES
      ENDIF
      step = 6.28319/FLOAT(ncord)
      cosst = COS(step)
      sinst = SIN(step)
      DO 20 i = nstart, ntimes
        x0 = XC + 0.01*FLOAT(i)*cosan
        y0 = YC + 0.01*FLOAT(i)*sinan
        CALL PLOT(x0,y0,3)
        x = x0
        y = y0
        npt = ncord - 1
        DO 10 j = 1, npt
          x1 = XC + (x-XC)*cosst - (y-YC)*sinst
          y1 = YC + (x-XC)*sinst + (y-YC)*cosst
          CALL PLOT(x1,y1,2)
          x = x1
          y = y1
   10   CONTINUE
        CALL PLOT(x0,y0,2)
   20 CONTINUE
      CALL PLOT(XC,YC,3)
C
      RETURN
      END
C
C gmark1
C
      SUBROUTINE GMARK1(MARK,HGT,PX,PY)
C
C     + + + PURPOSE + + +
C     This routine is used to place the specified marker of specified
C     height at the specified location.  Valid marker types are referenced
C     by an integer number from 1-31 (for centered marker) or 32-133 (for
C     centered character, 32-126=ASCII character, space=32,0=48,A=65,a=97).
C     Valid marker types are defined in routines GPUTMK.
C
C     + + + KEYWORDS + + +
C     marker, symbol
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER MARK
      REAL HGT, PX, PY
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MARK   - Integer index of marker symbol.  Markers 1-5 are the
C              standard GKS markers.
C     HGT    - Height of marker in world coordinates, the width of
C              marker is assumed to be the same.
C     PX     - X location in world coordinates of center of marker.
C     PY     - Y location in world coordinates of center of marker.
C
C     + + + LOCAL VARIABLES + + +
      INTEGER isymb(1), nn
      REAL pxa(1), pya(1), ang, height
C
C     + + + INTRINSICS + + +
      INTRINSIC ABS
C
C     + + + EXTERNALS + + +
      EXTERNAL GPUTMK
C
C     + + + END SPECIFICATIONS + + +
C
      IF (MARK.NE.0) THEN
        isymb(1) = MARK
        height = ABS(HGT)
        pxa(1) = PX
        pya(1) = PY
        nn = 1
        ang = 0.0
        CALL GPUTMK(isymb,height,pxa,pya,nn,ang)
      ENDIF
C
      RETURN
      END
C
C     gmarkn.f
C
      SUBROUTINE GMARKN(MARK,HGT,PX,PY,N,ANGLE)
C
C     + + + PURPOSE + + +
C     This routine is used to place the specified marker(s) of specified
C     height at the specified location(s) at the specified angle of rotation.
C     GMARKN can place a "curve" of markers at the specified locations in
C     arrays PX and PY.  Valid marker types are referenced by an integer
C     number from 1-31 (for centered marker) or 32-133 (for centered
C     character, 32-126=ASCII character, space=32,0=48,A=65,a=97).  Valid
C     marker types are defined in routines GPUTMK.
C
C     + + + KEYWORDS + + +
C     marker, curve, rotate, symbol
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER MARK, N
      REAL HGT, PX(N), PY(N), ANGLE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MARK   - Integer index of marker symbol.  Markers 1-5 are the
C              standard GKS markers.
C     HGT    - Height of marker in world coordinates, the width of
C              marker is assumed to be the same.
C     PX     - Array of X location(s) in world coordinates of center of
C              marker (REAL). (dimensioned at least N)
C     PY     - Array of Y location(s) in world coordinates of center of
C              marker (REAL). (dimensioned at least N)
C     N      - Number of markers to produce.
C     ANGLE  - Angle of rotation of marker in clockwise direction (0.0
C              is normal orientation).
C
C     + + + LOCAL VARIABLES + + +
      INTEGER isymb(1), nn
      REAL height, ang
C
C     + + + INTRINSICS + + +
      INTRINSIC ABS
C
C     + + + EXTERNALS + + +
      EXTERNAL GPUTMK
C
C     + + + END SPECIFICATIONS + + +
C
      IF (MARK.NE.0) THEN
        nn = N
        height = ABS(HGT)
        ang = ANGLE
        isymb(1) = MARK
        CALL GPUTMK(isymb,height,PX,PY,nn,ang)
      ENDIF
C
      RETURN
      END
C
C     gmarks.f
C
      SUBROUTINE GMARKS(MARK,HGT,PX,PY,N,ANGLE)
C
C     + + + PURPOSE + + +
C     This routine is used to place the specified marker(s) of specified
C     height at the specified location(s) at the specified angle of rotation.
C     GMARKS can place a "curve" of markers, as specified in the array MARK,
C     at the specified locations in arrays PX and PY.  For example, a different
C     marker type can be placed at each location.  Valid marker types are
C     referenced by an integer number from 1-31 (for centered marker) or 32-128
C     (for centered character, 32-126=ASCII character, space=32,0=48,A=65,a=97).
C     Valid marker types are defined in routine GPUTMK.
C
C     + + + KEYWORDS + + +
C     marker, curve, rotate, symbol
C
C     + + + DUMMY ARGUMENTS + + +
      INTEGER N
      INTEGER MARK(N)
      REAL HGT, PX(N), PY(N), ANGLE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     MARK   - Array of integer indices of the marker symbol to be placed
C              at each location.  Markers 1-5 are the standard GKS markers.
C     HGT    - Height of marker in world coordinates, the width of
C              marker is assumed to be the same.
C     PX     - Array of X location(s) in world coordinates of center of
C              marker (REAL). (dimensioned at least N)
C     PY     - Array of Y location(s) in world coordinates of center of
C              marker (REAL). (dimensioned at least N)
C     N      - Number of markers to produce.
C     ANGLE  - Angle of rotation of marker in clockwise direction (0.0
C              is normal orientation).
C
C     + + + LOCAL VARIABLES + + +
      INTEGER nn
      REAL height, ang
C
C     + + + INTRINSICS + + +
      INTRINSIC SIGN
C
C     + + + EXTERNALS + + +
      EXTERNAL GPUTMK
C
C     + + + END SPECIFICATIONS + + +
C
      nn = N
      height = SIGN(HGT,-1.0)
      ang = ANGLE
C
      CALL GPUTMK(MARK,height,PX,PY,nn,ang)
C
      RETURN
      END
C
C     gmkstr.f
C
      SUBROUTINE GMKSTR(STRING,NCHAR,HGT,PX,PY,ANGLE)
C
C     + + + PURPOSE + + +
C     This routine is used to place the specified string of specified
C     height at the specified location using the software generated
C     characters.  Valid characters are defined in routines GPUTMK.
C
C     + + + KEYWORDS + + +
C     string, character
C
C     + + + DUMMY ARGUMENTS + + +
      CHARACTER*(*) STRING
      INTEGER NCHAR
      REAL HGT, PX, PY, ANGLE
C
C     + + + ARGUMENT DEFINITIONS + + +
C     STRING - Character string to plot.
C     NCHAR  - Number of character in string to plot.
C     HGT    - Height of character(s) in world coordinates, the width is
C              assumed to be the same.
C     PX     - X location in world coordinates of lower-left corner of marker
C     PY     - Y location in world coordinates of lower-left corner of marker
C     ANGLE  - Angle of rotation of marker in clockwise direction (0.0
C              is normal orientation).
C
C     + + + LOCAL VARIABLES + + +
      INTEGER isymb(1), nn, i
      REAL pxa(1), pya(1), ang, height, c, s, xm, ym
C
C     + + + INTRINSICS + + +
      INTRINSIC ABS, COS, SIN, ICHAR
C
C     + + + EXTERNALS + + +
      EXTERNAL GPUTMK
C
C     + + + END SPECIFICATIONS + + +
C
      nn = NCHAR
      height = ABS(HGT)
      ang = ANGLE*0.0174533
      c = COS(ang)
      s = SIN(ang)
      xm = height*c
      ym = height*s
      pxa(1) = PX + height/2.0
      pya(1) = PY + height/2.0
      ang = ANGLE
      DO 10 i = 1, nn
        isymb(1) = ICHAR(STRING(i:i))
        IF (isymb(1).NE.0) CALL GPUTMK(isymb,height,pxa,pya,1,ang)
        pxa(1) = pxa(1) + xm
        pya(1) = pya(1) + ym
   10 CONTINUE
C
      RETURN
      END
C
C     gdgrot.f
C
      SUBROUTINE GDGROT(ANGLE,X,Y)
C
C     + + + PURPOSE + + +
C     Rotates the X and Y coordinates.  This routine is called by GPUTMK.
C
C     + + + KEYWORDS + + +
C     rotate
C
C     + + + DUMMY ARGUMENTS + + +
      REAL ANGLE, X, Y
C
C     + + + ARGUMENT DEFINITIONS + + +
C     ANGLE  - The angle of rotation.
C     X      - The X coordinate.
C     Y      - The Y coordinate.
C
C     + + + LOCAL VARIABLES + + +
      REAL t, cosang, sinang
C
C     + + + INTRINSICS + + +
      INTRINSIC COS, SIN
C
C     + + + END SPECIFICATIONS + + +
C
C      Rotate X and Y:
C
      t = 0.0174533*ANGLE
      cosang = COS(t)
      sinang = SIN(t)
 
      t = X*cosang - Y*sinang
      Y = Y*cosang + X*sinang
      X = t
C
      RETURN
      END
